/*

  root2ps

  Reads root file and prints all TCanvas,
  TH1 and TH2 to Postscript file

  (c) Bjrn Spruck

  created 17.3.2007

  (code based on hadd sourcecode)

*/

#include "RConfig.h"
#include <string>
#include "TROOT.h"
#include "TChain.h"
#include "TCanvas.h"
#include "TFile.h"
#include "TH1.h"
#include "TH2.h"
#include "TKey.h"
#include "TObjString.h"
#include "Riostream.h"

#include "regex.h"
#include "fnmatch.h"
#include <getopt.h>

void root2ps(TString source, TString target);

char *wildcard=0;
regex_t *re=0;
bool rekursiv=true;
bool verbose=false;

void usage(int argc, char **argv )
{
	cout <<
		 "Usage: " << argv[0] << " [-v] [-r] [-m regex] [-w wildcards] rootfile [psfile]\n"
		"Print all (matching) TCanvas, TH1, TH2 into a ps file.\n"
		"\nOptions:\n"
		"  -v          : verbose output\n"
		"  -r          : recursive into subdiretories (inside the root file) (default)\n"
		"  -n          : NOT recursive\n"
		"  -m regex    : plot only objects fitting regular expression (.*, MyHist.*, MyHist0.fit)\n"
		"  -w wildcard : plot only objects fitting wildcard (*, MyHist*, MyHist0?fit)\n"
		"\nThings to know:\n"
		"  ~/.root2ps/rootlogon.C will be executed before doing anything.\n"
		"default is project all objects\n"
		"  TH2 will use Draw(\"colz!\").\n"
		<<endl;
}

int main( int argc, char **argv )
{
	TString sname, tname;

  static struct option long_options[] = {
    {"help", 0, 0 , '?'},
    {0, 0, 0, 0}
  };
  int option_index = 0;
	char *regex=0;// ".*";// Default: anything

	if( argc<=1){
		usage(argc,argv);
    return 1;
	}
  Int_t c;
  while ((c = getopt_long (argc, argv,
		":hrnv"
		"m:w:",
		long_options, &option_index)) != -1)
	{
    switch(c){
      case Int_t(':'):
        cerr << "Missing argument to \""<<optarg<<"\"!"<<endl;
        return 1;
      case Int_t('h'):
      case Int_t('?'):
				usage(argc,argv);
        return 1;

      case Int_t('v'):
				verbose=true;
				break;

      case Int_t('m'):
				wildcard=0;
        regex=optarg;
        cout << "regex: "<<regex<<endl;
				break;
			case Int_t('w'):
				regex=0;
        wildcard=optarg;
        cout << "wildcard: "<<wildcard<<endl;
        break;
      case Int_t('r'):
				rekursiv=true;
        break;
      case Int_t('n'):
				rekursiv=false;
        break;
    }
  }

  if (optind == (argc-2)){
		sname=argv[optind];
		tname=argv[optind+1];
  }else if (optind == (argc-1)){
		sname=argv[optind];
		tname=sname+TString(".ps");
	}else{
		cerr<< "Somethings missing..."<<endl;
		return(1);
	}

	cout << "Process ~/.root2ps/rootlogon.C"<<endl;
	gROOT->ProcessLine(".L ~/.root2ps/rootlogon.C");
	gROOT->ProcessLine("rootlogon()");
	cout << "Logon finished"<<endl;

	if(verbose){
		cout << "Verbose is on" << endl;
		cout << "Recursiv is " <<(rekursiv?"on":"off")<< endl;
		if(regex) cout << "Regex: "<< regex << endl;
		if(wildcard) cout << "Wildcard: "<< wildcard << endl;
	}

	cout << "Source file (root): " << sname << endl;
	cout << "Target file (ps): " << tname << endl;

	if( regex){
		re=new regex_t;
		if(regcomp(re, regex, 0)) re=0;
	}

	root2ps( sname,tname);

	return 0;
}

void loop_in_dir(TDirectory *Source,TString tname,TCanvas *cc)
{
	// loop over all keys in this directory
	TIter nextkey( Source->GetListOfKeys() );
	TKey *key;

	while ( (key = (TKey*)nextkey()))
	{
		// read object from first source file
		Source->cd();
		TObject *obj = key->ReadObj();

		if ( obj->IsA()->InheritsFrom( "TDirectory" ) ) {
			// it's a subdirectory
			if(verbose) cout << "Found subdirectory: " << obj->GetName() << endl;
			loop_in_dir((TDirectory *)obj,tname,cc);
		} else {// end if is a directory
			if( (re && regexec(re,obj->GetName(),0,NULL,0)==0)
					|| (wildcard && fnmatch(wildcard, obj->GetName(), FNM_EXTMATCH)==0)
					|| (re==0 && wildcard==0)
			){
				if ( obj->IsA()->InheritsFrom( "TCanvas" ) ) {
					TCanvas *c;
					c=(TCanvas *)obj;
					if(verbose) cout << "Found TCanvas: " << c->GetName() << endl;
					c->Print(tname);
				}else if ( obj->IsA()->InheritsFrom( "TH2" ) ) {
					TH2 *c;
					c=(TH2 *)obj;
					if(c->GetEntries()>0){
						if(verbose) cout << "Found TH2: " << c->GetName() << endl;
						TCanvas *dd;
						dd=new TCanvas;
						c->Draw("colz");
						dd->SetName(c->GetName());
						dd->Print(tname);
						delete dd;
					}
				}else if ( obj->IsA()->InheritsFrom( "TH1" ) ) {
					TH1 *c;
					c=(TH1 *)obj;
					if(c->GetEntries()>0){
						if(verbose) cout << "Found TH1: " << c->GetName() << endl;
						TCanvas *dd;
						dd=new TCanvas;
						c->Draw();
						dd->SetName(c->GetName());
						dd->Print(tname);
						delete dd;
					}
				}
			}// end if regex or wildcard fit
		}// end if not a directory
	} // while ( ( TKey *key = (TKey*)nextkey() ) )
}

void root2ps(TString sname,TString tname)
{
	TFile *Source;

	Source = TFile::Open( sname, "");
	if (!Source || Source->IsZombie()) {
		cerr << "Error opening source file (does " << sname << " exist?)." << endl;
		exit(1);
	}

   //gain time, do not add the objects in the list in memory
	TH1::AddDirectory(kFALSE);
	TH2::AddDirectory(kFALSE);

	TCanvas *cc=new TCanvas();
	cc->Print(tname+"[");

	loop_in_dir(Source, tname, cc);

	cc->Print(tname+"]");
	delete cc;
}
